//------------------------------------------
// Includes
//------------------------------------------
#include "spi.h"

//------------------------------------------
// SPI-Initialiserungsroutine
//------------------------------------------
void spi_init()
{
	DDR_SPI	|= (1<<MOSI) | (1<<SCK);	// MOSI,SCK als Output
	DDR_SPI	&= ~(1<<MISO);				// MISO als Input
	DDR_SR |= (1<<nCSi) |(1<<nCSo);
	PORT_SPI|= (1<<MISO);				// MISO mit internem pullup
	PORT_SR	|= (1<<nCSo) | (1<<nCSi);	// Schieberegister disable
	sr_in	= 0;						// Schieberegister-Buffer initialisieren
	sr_out	= 0;
	SPCR = (1<<SPE) | (1<<MSTR) | (1<<CPOL);	// SPI enable, LSB first, Master, SPI Mode 3
	SPSR = (1<<SPI2X);							// max. SPI-Geschwindigkeit (fosc/2)
}


//------------------------------------------
// Liest die Lichtschranken-Schieberegister aus
//------------------------------------------
unsigned int sr_get()
{
	unsigned int ret = 0;
	uint8_t temp = 0;

	cli();						// global interrupt disable
	SPCR &= ~(1<<CPHA);			// triggern mit negativer Flanke
	PORT_SR &= ~(1<<nCSi);		// bernhemen der Daten in den seriellen Buffer im 74165
	PORT_SR |= (1<<nCSi);		// 
	SPDR = 0;					// Takt erzeugen
	while(!(SPSR & (1<<SPIF)));	// Warte bis Byte gesendet
	temp = SPDR;				// gelesenes Byte auslesen
	SPDR = 0;					// Takt fr das 2. Byte erzeugen
	ret |= (unsigned int)temp;	// ins Low-Byte schreiben
	ret <<= 8;					// ins HighByte schieben
//	ret &= 0xff00;				// Low-Byte lschen
	while(!(SPSR & (1<<SPIF)));	// Warte bis Byte gesendet
	temp = SPDR;				// gelesenes Byte auslesen
	ret |= (unsigned int)temp;	// Low-Byte stellen
	sei();						// global interrupt enable

	return ret;
}


//------------------------------------------
// Setzt das Schieberegister fr Spulen
//------------------------------------------
void sr_set(uint8_t data)
{
	cli();						// global interrupt disable
	SPCR |= (1<<CPHA);			// triggern mit positiver Flanke	
	SPDR = data;				// zu sendendes Byte
	while(!(SPSR & (1<<SPIF)));	// Warte bis Byte gesendet
	PORT_SR |= (1<<nCSo);		// bernahme der Daten an Ausgang am 74595
	PORT_SR &= ~(1<<nCSo);		// 
	sei();						// global interrupt enable
}
