//------------------------------------------
// Includes
//------------------------------------------
#include "uart.h"

//------------------------------------------
// UART-Initialiserungsroutine
//------------------------------------------
void uart_init()
{
	UBRR0 = UART_BAUD_CALC(UART_BAUD_RATE,F_CPU);					// set Baud-Rate
	UCSR0B = (1<<RXEN0) | (1<<TXEN0) | (1<<RXCIE0);					// Enable Receiver and Transmitter
	UCSR0C = (1<<UCSZ00) | (3<<UCSZ01);								// set Frame-Format (8Bit, no parity, 1 stop bit)
   	
	// UART-struct initialisieren
	uart0.in_ptr		= 0;
	uart0.out_ptr		= 0;
	uart0.in_payload	= 0;
	uart0.out_payload	= 0;
   	uart0.send 			= false;			
   	uart0.rec_complete 	= false;		
}


//------------------------------------------
// Fgt dem Sende-Buffer einen String hinzu
//------------------------------------------
bool uart_puts(uint8_t* data)
{
	if (uart0.send == false)			// wenn Sende-Routine sendebereit
	{
		strcpy(uart0.out_buffer, data);	// kopiere zu sendende Daten in Sende-Buffer
		uart0.send = true;				// setze Flag zum Senden
		return true;
	}
	else
		return false;					// wenn Sende-Routine nicht sendebereit -> return false
}


//------------------------------------------
// liest einen String aus dem Empfangsbuffer aus
//------------------------------------------
bool uart_gets(uint8_t* data)
{
	if (uart0.rec_complete == true)		// wenn etwas komplett empfangen wurde
	{
		strcpy(data, uart0.in_buffer);	// Daten aus Empfangsbuffer kopieren
		uart0.rec_complete = false;		// Flag lschen
		return true;
	}
	else
		return false;					// wenn nichts empfangen wurde, return false
}


//------------------------------------------
// Receive Complete ISR
//------------------------------------------
SIGNAL (SIG_USART_RECV)
{
	uint8_t data, i;	// Hilfsvariable

	cli();	// global interrupt disable

	data = UDR0;	// Byte aus UART-Empfangsregister auslesen
	
	if (uart0.in_ptr == 0)
	{
		if (data == STX)			// Frameanfang erkennen STX
			uart0.in_ptr++;			// Zhler weitersetzen
	}
		
	else if (uart0.in_ptr == 1)
	{
		uart0.in_payload = data;	// Payload bernehmen
		uart0.in_ptr++;				// Zhler weitersetzen
	}

	else
	{				
		if ( uart0.in_ptr < (uart0.in_payload+3) )			// wenn payload noch nicht berschritten
		{
			uart0.in_buffer[(uart0.in_ptr-2)] = data;		// Daten in Empfangsbuffer bernehmen (auch Checksumme)
			uart0.in_ptr++;
		}
		else if (data == ETX)								// Wenn Frameende ETX
		{
			data = 0;
			for (i = 0; i < uart0.in_payload; i++) 			// Checksumme berprfen
			{
				data += uart0.in_buffer[i];					// additive Checksumme
			}
			if ( data == uart0.in_buffer[uart0.in_payload] )// Wenn Checksumme korrekt (Ckecksumme an letzter Stelle im Bufer)
			{
				uart0.in_buffer[uart0.in_payload] = '\0';	// String-Abschlusszeichen einfgen
				uart0.in_ptr = 0;							// Zhler zurcksetzen
				uart0.rec_complete = true;					// Flag fr empfangene Daten setzen
			}
			else
				uart0.in_ptr=0;	// falsche Checksumme
		}
		else
		{
			uart0.in_ptr = 0;	// falscher Frame
		}
	}

	sei();	// global interrupt enable
}


//------------------------------------------
// Data Register Empty ISR
//------------------------------------------
SIGNAL (SIG_USART_DATA)
{
	uint8_t data = 0,i;	// Hilfsvariablen

	cli();	// global interrupt disable

	if (uart0.send == true)
	{
		if (uart0.out_ptr == 0)	// Wenn noch nichts gesendet
		{
			UDR0 = STX;			// Sende Frameanfang STX
			uart0.out_ptr++;	// erhhe Zhler
		}

		else if (uart0.out_ptr == 1)	// Wenn Frame-Startzeichen gesendet
		{
			UDR0 = uart0.out_payload;	// Payload senden
			uart0.out_ptr++;			// erhhe Zhler
		}

		else if (uart0.out_ptr < (uart0.out_payload+2) )	// Wenn Daten gesendet
		{
			UDR0 = uart0.out_buffer[uart0.out_ptr-2];
			uart0.out_ptr++;
		}
		else if (uart0.out_ptr == (uart0.out_payload+2) )
		{
			for (i = 0; i < uart0.out_payload; i++) // additive Checksumme bilden
			{
				data += uart0.out_buffer[i];	
			}	
			
			UDR0 = data;
			uart0.out_ptr++;
		}

		else if (uart0.out_ptr == (uart0.out_payload+3) )	// wenn Cheksumme gesendet
		{
			UDR0 = ETX;									// Sende Frameende ETX
			uart0.out_ptr = 0;							// Zhler zurcksetzen
			uart0.send = false;							// Senden-Flag zurcksetzen
			uart0.out_payload = 0;						// Payload zurcksetzen
			UCSR0B &= ~(1<<UDRIE0);						// UART Interrupt disable
		}
	}
	sei();	// global interrupt enable
}
