#ifndef INC_UART_H
#define INC_UART_H

//-------------------------------------------------
// Includes
//-------------------------------------------------
#include <avr/io.h>
#include <avr/interrupt.h>
#include <avr/signal.h>
#include <string.h>
#include <stdbool.h>

//-------------------------------------------------
// Defines
//-------------------------------------------------
#define UART_BAUD_RATE	19200
#define F_CPU			7370000
#define STX				2
#define ETX				3


//-------------------------------------------------
// Makros
//-------------------------------------------------
#define UART_BAUD_CALC(UART_BAUD_RATE,F_OSC) ((F_CPU)/((UART_BAUD_RATE)*16L)-1)


//-------------------------------------------------
// Typdefinitionen
//-------------------------------------------------
typedef struct uart			// beinhaltet den Sende/Empfangs-Buffer mit dazugehrigen Flags und Pointern
{
   uint8_t 	in_buffer[25];	// Empfangsbuffer
   uint8_t 	out_buffer[25];	// Sendebuffer
   uint8_t	out_ptr;		// Zhlt gesendete Zeichen mit
   uint8_t	in_ptr;			// Zhlt empfangene Zeichen mit   
   uint8_t	in_payload;		// Anzahl der Daten-Zeichen im Empfangsframe
   uint8_t	out_payload;	// Anzahl der Daten-Zeichen im zu sendenden Frame
   bool		send;			// Flag, gesetzt wenn etwas zu senden ist
   bool		rec_complete;	// Flag, gesetzt wenn etwas empfangen wurde
}uart;


//-------------------------------------------------
// Variablen
//-------------------------------------------------
struct uart uart0;	// Instanz der UART


//-------------------------------------------------
// Funktions-Prototypen
//-------------------------------------------------
void	uart_init	();	// initialisiert UART mit den bentigten Interrupts
						// Senden und Empfangen wird von den Interrupts erledigt
						// es brauchen nur lediglich der Sendebuffer gefllt, die Pointer  
						// und das send-Flag gesetzt werden

bool	uart_puts	(uint8_t*);	// fgt dem Sendebuffer ein String hinzu (abschliessend \0)
bool	uart_gets	(uint8_t*);	// liest einen String aus (ebenfalls abschliessende \0)

#endif

